﻿Public Class methods

    Public Shared Sub drawGrid(ByVal g As Graphics, ByVal f As Font)
        g.DrawRectangle(Pens.LightGray, New Rectangle(25, 25, 400, 400))

        For c As Integer = 50 To 400 Step 25
            g.DrawLine(Pens.LightGray, c, 25, c, 425)
        Next
        For r As Integer = 50 To 400 Step 25
            g.DrawLine(Pens.LightGray, 25, r, 425, r)
        Next

        g.DrawLine(Pens.Black, 225, 25, 225, 425)
        g.DrawLine(Pens.Black, 25, 275, 425, 275)

        Dim lblWidth As Integer

        For x As Integer = -8 To 8
            lblWidth = CInt(g.MeasureString(x.ToString, New Font(f.FontFamily, 7)).Width)
            g.DrawString(x.ToString, New Font(f.FontFamily, 7), Brushes.Black, 25 + ((x + 8) * 25) - lblWidth, 276)
        Next

        For y As Integer = -6 To 10
            lblWidth = CInt(g.MeasureString(y.ToString, New Font(f.FontFamily, 7)).Width)
            g.DrawString(y.ToString, New Font(f.FontFamily, 7), Brushes.Black, 225 - lblWidth, 426 - (y + 6) * 25)
        Next
    End Sub

    Public Shared Sub drawShape(ByVal g As Graphics, ByVal vertices As List(Of Point))
        If vertices.Count > 2 Then
            g.DrawPolygon(New Pen(Color.Red, 2), vertices.ToArray)
        Else
            For x As Integer = 0 To vertices.Count - 1
                g.FillEllipse(Brushes.Red, New Rectangle(vertices(x).X - 3, vertices(x).Y - 3, 6, 6))
            Next
        End If
    End Sub

    Public Shared Sub drawEnlargedShape(ByVal g As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal s As Decimal, ByVal polarity As String, ByVal vertices As List(Of Point))
        If vertices.Count >= 3 Then
            Dim pen As New Pen(Color.LimeGreen)
            pen.DashStyle = Drawing2D.DashStyle.Dash

            Dim newPoints As New List(Of Point)()

            For x As Integer = 0 To vertices.Count - 1
                Dim distance As Single = measurement.lineLength(New Point(l, t), vertices(x))
                Dim angle As Double = (If(polarity.Equals("+"), 360, 180)) - angles.FindAngle(New Point(l, t), vertices(x))

                Dim scaleDistance As Decimal = CDec(distance * s)

                'Turn degrees to radians (because of the sin and cos operations)
                Dim angleRadians As Double = angle * (Math.PI / 180)

                'Calculate X1 And Y1
                Dim pointX1 As Integer = CInt(l + Math.Cos(angleRadians) * scaleDistance)
                Dim pointY1 As Integer = CInt(t - Math.Sin(angleRadians) * scaleDistance)

                newPoints.Add(New Point(pointX1, pointY1))
            Next

            If polarity.Equals("+") Then
                For x As Integer = 0 To vertices.Count - 1
                    g.DrawLine(pen, l, t, If(s > 1D, newPoints(x).X, vertices(x).X), If(s > 1D, newPoints(x).Y, vertices(x).Y))
                Next
            Else
                For x As Integer = 0 To vertices.Count - 1
                    g.DrawLine(pen, l, t, vertices(x).X, vertices(x).Y)
                    g.DrawLine(pen, l, t, newPoints(x).X, newPoints(x).Y)
                Next
            End If

            Dim pen2 As New Pen(Color.DodgerBlue)
            pen2.DashStyle = Drawing2D.DashStyle.Dash

            g.DrawPolygon(pen2, newPoints.ToArray)
        End If
    End Sub

    Public Shared Sub drawBorders(ByVal g As Graphics, ByVal c As Color)
        g.FillRectangle(New SolidBrush(c), New Rectangle(0, 0, 25, 500))
        g.FillRectangle(New SolidBrush(c), New Rectangle(426, 0, 25, 500))
        g.FillRectangle(New SolidBrush(c), New Rectangle(0, 0, 450, 25))
        g.FillRectangle(New SolidBrush(c), New Rectangle(0, 426, 450, 75))
    End Sub

    Public Shared Sub drawCross(ByVal g As Graphics, ByVal l As Integer, ByVal t As Integer)
        Dim p As New Pen(Color.LimeGreen, 2)
        g.DrawLine(p, l, t - 10, l, t + 10)
        g.DrawLine(p, l - 10, t, l + 10, t)
    End Sub

    Public Shared Sub drawHighlight(ByVal g As Graphics, ByVal highlight As Point)
        If highlight <> Nothing Then
            Dim shadowPath As New Drawing.Drawing2D.GraphicsPath
            shadowPath.AddEllipse(New Rectangle(highlight.X - 10, highlight.Y - 10, 20, 20))
            Using pgb As New Drawing2D.PathGradientBrush(shadowPath)
                'Normal
                pgb.CenterColor = Color.Red  'This is the real one
                pgb.SurroundColors = New Color() {Color.Transparent}
                pgb.FocusScales = New PointF(0.1F, 1.0F)

                g.FillPath(pgb, shadowPath)
            End Using
        End If
    End Sub

    Public Shared Sub drawMirrorLine(ByVal g As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal v As Integer, ByVal x As Integer)
        Dim p As New Pen(Color.LimeGreen, 2)
        p.DashStyle = Drawing2D.DashStyle.Dash

        Dim diagonals() As Integer = {5, 4, 3, 2, 1, 0, 1, 2, 3, 4, 5}

        Dim x1 As Integer
        Dim y1 As Integer
        Dim x2 As Integer
        Dim y2 As Integer

        Select Case x
            Case 0 'Vertical
                g.drawLine(p, l, 25, l, 425)
            Case 1 'Horizontal
                g.drawLine(p, 25, t, 425, t)
            Case 2 'Diagonal - SW to NE
                If v <= 2 Then
                    x1 = 25 + diagonals(v + 3) * 25
                    y1 = 425
                    x2 = 425
                    y2 = 25 + diagonals(v + 3) * 25
                Else
                    x1 = 25
                    y1 = 425 - diagonals(v + 3) * 25
                    x2 = 425 - diagonals(v + 3) * 25
                    y2 = 25
                End If
                g.drawLine(p, x1, y1, x2, y2)
            Case 3 'Diagonal - NW to SE
                If v <= 2 Then
                    x1 = 25
                    y1 = 25 + diagonals(v + 3) * 25
                    x2 = 25 + (16 - diagonals(v + 3)) * 25
                    y2 = 425
                Else
                    x1 = 25 + diagonals(v + 3) * 25
                    y1 = 25
                    x2 = 425
                    y2 = 25 + (16 - diagonals(v + 3)) * 25
                End If
                g.drawLine(p, x1, y1, x2, y2)
        End Select

    End Sub

    Public Shared Sub drawMirrorShape(ByVal g As Graphics, ByVal vertices As List(Of Point), ByVal v As Integer, ByVal x As Integer)
        If vertices.Count < 3 Then
            Return
        End If

        Dim pen As New Pen(Color.DodgerBlue, 2)
        pen.DashStyle = Drawing2D.DashStyle.Dash

        Dim gridPoints As New List(Of Point)
        For Each p As Point In vertices
            gridPoints.Add(New Point(pointMethods.X2c(p.X), pointMethods.Y2r(p.Y)))
        Next p

        Dim reflectionPoints As New List(Of Point)
        Select Case x
            Case 0 'Vertical
                For Each p As Point In gridPoints
                    reflectionPoints.Add(New Point(pointMethods.swapPolarity(p.X - v) + v, p.Y))
                Next p
            Case 1 'Horizontal
                For Each p As Point In gridPoints
                    reflectionPoints.Add(New Point(p.X, pointMethods.swapPolarity(p.Y - v) + v))
                Next p
            Case 2 'Diagonal - SW to NE
                For Each p As Point In gridPoints
                    reflectionPoints.Add(New Point(p.Y - v, p.X + v))
                Next p
            Case 3 'Diagonal - NW to SE
                For Each p As Point In gridPoints
                    reflectionPoints.Add(New Point(pointMethods.swapPolarity(p.Y - v), pointMethods.swapPolarity(p.X) + v))
                Next p
        End Select

        Dim reflectionCoordinates As New List(Of Point)
        For Each p As Point In reflectionPoints
            reflectionCoordinates.Add(New Point(pointMethods.c2X(p.X), pointMethods.r2Y(p.Y)))
        Next

        g.DrawPolygon(pen, reflectionCoordinates.ToArray)
        
    End Sub

    Public Shared Sub drawRotatedShape(ByVal g As Graphics, ByVal vertices As List(Of Point), ByVal r As Integer, ByVal l As Integer, ByVal t As Integer, ByVal polarity As String)
        If vertices.Count < 3 Then
            Return
        End If

        Dim pen As New Pen(Color.DodgerBlue, 2)
        pen.DashStyle = Drawing2D.DashStyle.Dash

        Dim gridPoints As New List(Of Point)
        For Each p As Point In vertices
            gridPoints.Add(New Point(pointMethods.X2c(p.X), pointMethods.Y2r(p.Y)))
        Next p

        l = pointMethods.X2c(l)
        t = pointMethods.Y2r(t)

        Dim rotationPoints As New List(Of Point)

        Select Case polarity & r.ToString()
            Case "+90", "-270"
                For Each p As Point In gridPoints
                    rotationPoints.Add(New Point((p.Y - t) + l, -(p.X - l) + t))
                Next p
            Case "-180", "+180"
                For Each p As Point In gridPoints
                    rotationPoints.Add(New Point(-(p.X - l) + l, -(p.Y - t) + t))
                Next p
            Case "+270", "-90"
                For Each p As Point In gridPoints
                    rotationPoints.Add(New Point(-(p.Y - t) + l, p.X - l + t))
                Next p
            Case Else
                For Each p As Point In gridPoints
                    rotationPoints.Add(New Point(p.X, p.Y))
                Next p
        End Select

        Dim rotationCoordinates As New List(Of Point)
        For Each p As Point In rotationPoints
            rotationCoordinates.Add(New Point(pointMethods.c2X(p.X), pointMethods.r2Y(p.Y)))
        Next

        g.DrawPolygon(pen, rotationCoordinates.ToArray)

    End Sub

    Public Shared Sub drawTranslatedShape(ByVal g As Graphics, ByVal vertices As List(Of Point), ByVal l As Integer, ByVal t As Integer)
        If vertices.Count < 3 Then
            Return
        End If

        Dim pen As New Pen(Color.DodgerBlue, 2)
        pen.DashStyle = Drawing2D.DashStyle.Dash

        Dim gridPoints As New List(Of Point)
        For Each p As Point In vertices
            gridPoints.Add(New Point(pointMethods.X2c(p.X), pointMethods.Y2r(p.Y)))
        Next p

        l = pointMethods.X2c(l)
        t = pointMethods.Y2r(t)

        Dim translationPoints As New List(Of Point)

        For Each p As Point In gridPoints
            translationPoints.Add(New Point(p.X + l, p.Y + t))
        Next p

        Dim translationCoordinates As New List(Of Point)
        For Each p As Point In translationPoints
            translationCoordinates.Add(New Point(pointMethods.c2X(p.X), pointMethods.r2Y(p.Y)))
        Next

        g.DrawPolygon(pen, translationCoordinates.ToArray)

    End Sub


End Class
